syntax = "proto3";

package gitaly;

option go_package = "gitlab.com/gitlab-org/gitaly-proto/go/gitalypb";

import "shared.proto";

service StorageService {
  rpc ListDirectories(ListDirectoriesRequest) returns (stream ListDirectoriesResponse) {
    option (op_type) = {
      op: MUTATOR
      scope_level: SERVER,
    };
  }
  rpc DeleteAllRepositories(DeleteAllRepositoriesRequest) returns (DeleteAllRepositoriesResponse) {
    option (op_type) = {
      op: MUTATOR
      scope_level: SERVER,
    };
  }
}

message ListDirectoriesRequest {
  string storage_name = 1;
  uint32 depth = 2;
}

message ListDirectoriesResponse {
  repeated string paths = 1;
}

message DeleteAllRepositoriesRequest {
  string storage_name = 1;
}

message DeleteAllRepositoriesResponse {}
